from gtkmvc.controller import Controller
from gtk import main_quit
from gtk import ListStore
from gtk import CellRendererText
from gtk import TreeViewColumn
from birne_view import BirneSelectorView
from os import path
from os import environ
from os import stat
import gobject


class BirneController (Controller):
    """
    """

    def __init__(self, model):
        """Contructor. model will be accessible via the member 'self.model'.
        Registration is also performed."""
        Controller.__init__(self, model)
        self.model.registerObserver(self)

        # This is the current working directory for the file selector.
        # We have to remember this because the selector is instantiated
        # on every click on the add button.
        self.dir = str(environ['HOME'])

        return

    def registerView(self, view):
        Controller.registerView(self, view) # Calls the overloaded method

        # Connects the exiting signal:
        view.get_top_widget().connect("destroy", main_quit)
        view["quit"].connect("clicked", main_quit)

        #
        self.listModel = ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
        self.view["CDContent"].set_model(self.listModel)

        renderer = CellRendererText()
        nameColumn = TreeViewColumn("File", renderer, text=1)
        sizeColumn = TreeViewColumn("Size", renderer, text=0)
        self.view["CDContent"].append_column(nameColumn)
        self.view["CDContent"].append_column(sizeColumn)
        return

    # Signal (view -> self: hey, the user talks to us!)
    def on_add_clicked(self, button):
        print "on_addFile_clicked %s" % button
        selectorCtrl = BirneSelectorController(self.model, self.dir)
        selector = BirneSelectorView(selectorCtrl)
        return

    def on_remove_clicked(self, button):
        print "on_remove_clicked %s" % button
        return

    def on_clear_clicked(self, button):
        print "on_remove_allFiles_clicked %s" % button
        self.model.content = {}
        return

    def on_burn_clicked(self, button):
        print "C: burn!"
        self.model.burn()
        return

    def on_mode_toggled(self, button):
        self.model.mode = button.get_name()
        return

    def on_speed_value_changed(self, button):
        self.model.speed = button.get_value()

    def on_rewritable_toggled(self, button):
        self.model.rewritable = button.get_active()
        return

    def on_burnFree_toggled(self, button):
        self.model.burnFree = button.get_active()
        return

    def on_simulate_toggled(self, button):
        self.model.simulate = button.get_active()
        return

###

    # Observables notifications (model -> self: hey, I am morphing!)
    def property_speed_change_notification(self, model, old, new):
        """  """
        print "C: model property speed changed: %s" % new

        return

    def property_rewritable_change_notification(self, model, old, new):
        """  """
        print "C: model property rewritable toggled: %s" % new
        return

    def property_burnFree_change_notification(self, model, old, new):
        """  """
        print "C: model property burnFree toggled: %s" % new
        return

    def property_simulate_change_notification(self, model, old, new):
        """  """
        print "C: model property simulate toggled: %s" % new
        return

    def property_content_change_notification(self, model, old, new):
        """ Rebuilds the whole CDContent TreeView. """
        print "C: model property content changed: %s" % new
        self.listModel.clear()
        for k, v in self.model.content.items():
            size = stat(k)[6] / 1024
            self.listModel.append( (v, size ) )
        self.view["CDContent"].show()
        return

    def property_mode_change_notification(self, model, old, new):
        """  """
        print "C: model property mode changed: %s" % new
        if new == "copy":
            self.view["contentButtonBox"].foreach( lambda w: w.set_sensitive(False) )
            self.view["contentScrolledWindow"].set_sensitive(False)
        elif old == "copy":
            self.view["contentButtonBox"].foreach( lambda w: w.set_sensitive(True) )
            self.view["contentScrolledWindow"].set_sensitive(True)
        return

    def property_contentsize_change_notification(self, model, old, new):
        """  """
        displaysize = str ( self.model.contentsize / 1024 / 1024 ) + " MiB"
        self.view["size"].set_label(displaysize)
        return


    pass # end of class

class BirneSelectorController(Controller):

    def __init__(self, model, dir=str(environ['HOME'])):
        """Contructor. model will be accessible via the member 'self.model'.
        Registration is also performed."""
        Controller.__init__(self, model)
        self.model.registerObserver(self)
        self.dir = dir
        return

    def registerView(self, view):
        Controller.registerView(self, view) # Calls the overloaded method

        if path.isdir(self.dir):
            self.view["mainWindow"].set_current_folder(self.dir)
        return

    def on_close_clicked(self, button):
        self.view.get_top_widget().destroy()
        return

    def on_select_clicked(self, button):
        new_files = self.view["mainWindow"].get_filenames()
        strip_folder = self.view["mainWindow"].get_current_folder()
        is_relative = self.view["rel_paths"].get_active()
        self.model.add_files(new_files, is_relative, strip_folder)
        return

    def on_select_and_close_clicked(self, button):
        self.on_select_clicked(button)
        self.on_close_clicked(button)

    pass

# vim: expandtab ts=4
