from gtkmvc.model import Model
import os

class BirneModel (Model):
    """ Model for Birne's main GUI. It contains properties for all the visible
    widgets and contains real world logic. The dirty work of a super model is
    done by the superclass Model."""

    __properties__ = {
        "mode"       : "copy",         # type of disc
        "speed"      : 4,              # burning speed
        "rewritable" : False,          # multi-session?
        "burnFree"   : True,           # extra buffers?
        "simulate"   : False,          # do not actually do anything?
        "content"    : {},             # content of disc. keys are source
                                       # paths, values are target names
                                       # (leading parts may be stripped off)
        "contentsize": 0               # size of disc content
        }

    def __init__(self):
        Model.__init__(self)
        return

    def burn(self):
        """Executes the burning function appropriate for current mode."""
        method = getattr(self, "burn_%s" % self.mode, "burn_copy")
        method()
        return

    def burn_copy(self):
        print "burn copy"
        return

    def burn_audio(self):
        print "burn audio"
        return

    def burn_files(self):
        print "burn files"
        return

    def burn_image(self):
        print "burn image"
        return

    def add_files(self, new_files, is_relative=True, strip_path=""):
        """Adds list of files and directories to CD content. Traverses through
        directories contained in new_files and adds every single file with its
        complete path. If is_relative is True, strip_path will be truncated
        from the path."""

        # Get a copy of current CD content...
        new_content = self.content

        # ... get new files...
        for file in new_files:
            if os.path.isdir(file):
                filetree = os.walk(file)
                for branch in filetree:
                    path = branch[0]
                    for leaf in branch[2]:
                        abspath = os.path.join(path, leaf)
                        self.contentsize += os.stat(abspath)[6]
                        target = self.get_target(abspath, is_relative, strip_path)
                        new_content[abspath] = target
            else:
                self.contentsize += os.stat(file)[6]
                target = self.get_target(file, is_relative, strip_path)
                new_content[file] = target

        # ... and copy new_content back to current content. We need to directly
        # assign the new content to our observable property to trigger
        # notification messages for the observers!
        self.content = new_content
        return


    def get_target(self, file, is_relative=True, strip_path=""):
        """ Returns the target filename from an absolute source name. """
        if is_relative:
            return file.split(strip_path, 1)[1]
        else:
            return file
    pass


# vim: expandtab ts=4
